/*
     This program shows how to "explicitly" load a DLL during runtime.

     Notice that this file does not include my-library.h and the only
     direct reference to the library functions is to pass their
     (decorated) names as strings to the function GetProcAddress().

     Notice also that this program does not need any special
     linking instructions.
*/
#include <stdio.h>
#include <windows.h>

// function prototype for error reporting function
void printError(char*);

int y;

int main()
{
   HINSTANCE hinstDLL;  // handle to the DLL
   FARPROC fpn_f, fpn_g, fpn_h;

   printf("\nExplicitly loading my-library.dll.\n\n");

   // Explicitly load the DLL into memory using LoadLibrary() function.
   hinstDLL = LoadLibrary("my-library.dll");
   if (hinstDLL == 0) { printError("LoadLibrary"); return -1; }

   // Get pointers to the library functions using GetProcAddress() function.
   // (Notice that we use the undecorated names of these functions.)
   fpn_f = GetProcAddress(hinstDLL, "f");
   if (fpn_f == 0) { printError("GetProcAddress (f)"); return -1; }
   fpn_g = GetProcAddress(hinstDLL, "g");
   if (fpn_g == 0) { printError("GetProcAddress (g)"); return -1; }
   fpn_h = GetProcAddress(hinstDLL, "h");
   if (fpn_h == 0) { printError("GetProcAddress (h)"); return -1; }

   // Call the "library" functions. Notice that the pointers
   // returned by GetProcAddress() need to be cast to the
   // correct kind of function pointer for each function.
   y = ((int(*)(int))fpn_f)(1);
   printf("the return value from f() was %d\n", y);
   y = ((int(*)(int,int))fpn_g)(10, 20);
   printf("the return value from g() was %d\n", y);
   y = ((int(*)(int,int,int))fpn_h)(100, 200, 300);
   printf("the return value from h() was %d\n", y);

   printf("\n");

   // Print out the values of the pointers to the functions
   printf("the address of function f() is %#.8x\n", (int)fpn_f);
   printf("the address of function g() is %#.8x\n", (int)fpn_g);
   printf("the address of function h() is %#.8x\n", (int)fpn_h);

   return 0;
}


/*************************************************************************************
    This method is used to print out useful error messages for
    Win32 (but not C Library) function calls.
*/
void printError(char* functionName)
{   LPVOID lpMsgBuf;
    int error_no;
    error_no = GetLastError();
    FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
    );
    // Display the string.
    fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
    fprintf(stderr, (LPTSTR)lpMsgBuf);
    // Free the buffer.
    LocalFree( lpMsgBuf );
}//printError
