/*
    This file calls four functions, f(), g(), h(), and printf(), that are
    not defined here in this file. When this file is compiled, somehow the
    compiler has to be told enough about these functions so that the compiler
    can determine if the function calls are appropriate. The usual way to
    inform the compiler about undefined functions is to put the information
    that the compiler needs in a header file.

    To inform the compiler about a function, the header file can contain one
    of two things. The header file can actually contain the definition of the
    function, or the header can contain just a prototype for the function. If
    the header only provides a prototype (which is almost always the case),
    then it will be up to the linker to find a definition for the function in
    either an obj file (static linking) or a lib file for a DLL (dynamic linking).

    To statically link this program with the "library" functions, compile the
    source file my-library.c and then use the following link command.
      lcclnk my-library-test.obj my-library.obj

    To dynamically link this program with a DLL version of the "library", first
    compile the source file my-library.c. Then make a DLL using the command
      lcclnk -DLL -nounderscores my-library.obj my-library.def
    Then link this program's obj file to the DLL's lib file with the command
      lcclnk my-library-test.obj my-library.lib
*/
#include <windows.h>  // for Sleep()
#include <stdio.h>
#include "my-library.h"

int y;

int main()
{
   // Call the "library" functions.
   y = f(1);
   printf("the return value from f() was %d\n", y);
   y = g(10, 20);
   printf("the return value from g() was %d\n", y);
   y = h(100, 200, 300);
   printf("the return value from h() was %d\n", y);

   printf("\n");

   // If the "library" functions are statically linked,
   // then the following code will print the addresses
   // of where the functions are loaded. If the functions
   // are contained in a DLL, then the following code will
   // print the addresses of the "stub" functions that the
   // linker links to from the dll's lib file.
   printf("the address of function f() is %#.8x\n", (int)&f);
   printf("the address of function g() is %#.8x\n", (int)&g);
   printf("the address of function h() is %#.8x\n", (int)&h);

   printf("\n");

   // If the "library" functions are contained in a DLL,
   // then the following code will print out the actual
   // address of where the functions are loaded (instead
   // of the addresses of the stub functions). You can
   // double check these values using the program Depends.
   /*
   y = **(int**)(((char*)(&f))+2);  //get the destination address from a jmp instruction
   printf("the address of f() in the DLL is %#.8x\n", y);
   y = **(int**)(((char*)(&g))+2);
   printf("the address of g() in the DLL is %#.8x\n", y);
   y = **(int**)(((char*)(&h))+2);
   printf("the address of h() in the DLL is %#.8x\n", y);
   */

   // Use this loop to keep the program running if you want
   // to observe it's running process.
   //while(1) Sleep(1000);

   return 0;
}
